#=========================================================#
# onis 0.3.3                             2002-03-29 12:32 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# html.pm                                                 #
#=========================================================#

package html;

use strict;

use vars qw(
		$data %color
		$VERSION $AUTHOR
	    );

*data = *::data;
%color = %{$Config::config->{'color'}};

open (OUT, '> ' . $::output) or die $!;

# Generates the HTML header including the CSS information.
# Doesn't take any arguments
sub start
{
	my $generated_time = scalar localtime ();
	print OUT <<EOF;
<html>
<head>
<title>#$::channel statistics created by $::user</title>
<meta http-equiv="Pragma" content="no-cache">
<style type="text/css">
<!--
a { color: $color{'text'}->{'link'}; }
body { color: $color{'text'}->{'normal'}; background: $color{'background'}->{'normal'}; }
em { font-weight: bold; font-style: normal; text-decoration: none; }
h1 { color: $color{'text'}->{'header'}; background: $color{'background'}->{'normal'}; font-family: Verdana; font-size: 18pt; text-align: center; font-style: italic; font-weight: bold;}
h2 { color: $color{'text'}->{'header'}; background: $color{'background'}->{'normal'}; font-family: Verdana; font-size: 14pt; text-align: center; font-style: italic; font-weight: normal; }
p { color: $color{'text'}->{'normal'}; background: $color{'background'}->{'normal'}; font-size: 10pt; text-align: center; }
td { color: $color{'text'}->{'table'}{'normal'}; background: $color{'background'}->{'table'}{'normal'}; text-align: left; vertical-align: bottom; font-family: Verdana; font-size: 9pt; }
td.bar { color: $color{'text'}->{'table'}{'percent'}; background: $color{'background'}->{'normal'}; font-size: 7pt; text-align: center; }
td.numeration { color: $color{'text'}->{'table'}{'side'}; background: $color{'background'}->{'table'}{'side'}; font-size: 7pt; }
td.header { color: $color{'text'}->{'table'}{'header'}; background: $color{'background'}->{'table'}{'header'}; font-weight: bold; font-style: italic; text-align: center; font-size: 10pt; }
td.copyleft  { color: $color{'text'}->{'table'}{'side'}; background: $color{'background'}->{'table'}{'side'}; font-size: 8pt; vertical-align: top; text-align: left; }
td.copyright { color: $color{'text'}->{'table'}{'side'}; background: $color{'background'}->{'table'}{'side'}; font-size: 8pt; vertical-align: top; text-align: right; }
td.hour { color: $color{'text'}->{'table'}{'header'}; background: $color{'background'}->{'table'}{'footer'}; font-size: 7pt; text-align: center; }
td.invis { background: $color{'background'}->{'normal'}; font-size: 8pt; vertical-align: middle; }
td.quote { font-style: italic; }
//-->
</style>
</head>

<body>
<div align="center">
EOF
}

sub head
#
# this actually starts the document by printing a
# headline and explains the meaning of the
# different bars..
#
{
my $generated_time = scalar localtime ();
print OUT <<EOF;

<h1>#$::channel stats by $::user</h1>
<hr width="50%">
<p>Statistics generated on $generated_time</p>
<hr width="50%">

<hr width="95%">
<table border="0">
<tr>
<td width="180" class="invis"><img src="$Config::config->{'bar'}{'horizontal'}[0]" height="15" width="30" alt="Red"> Hours 0-5</td>
<td width="180" class="invis"><img src="$Config::config->{'bar'}{'horizontal'}[1]" height="15" width="30" alt="Green"> Hours 6-11</td>
<td width="180" class="invis"><img src="$Config::config->{'bar'}{'horizontal'}[2]" height="15" width="30" alt="Blue"> Hours 12-17</td>
<td width="180" class="invis"><img src="$Config::config->{'bar'}{'horizontal'}[3]" height="15" width="30" alt="Red"> Hours 18-24</td>
</tr>
</table>
<hr width="95%">

EOF
}

# this routine adds a box to the end of the html-
# page with onis' homepage URL, the author's name
# and email-address. Feel free to uncomment the
# creation of this box if it's appereance nags
# you..
sub copyleft
{
	my $runtime = time () - $::starttime;
	print "Runtime: $runtime seconds\n" if $::debug;
	my $now = scalar localtime ();
	print OUT <<EOF;
<!-- This script is under GPL (GNU public license). You may copy and modify it. -->
<br><br>

<table width="95%" border="0">
<tr><td>
<table width="100%"  border="0" cellpadding="5" cellspacing="0">
<tr><td class="copyleft">This page was generated <nobr>on $now</nobr> <nobr>with onis $::VERSION (&quot;onis not irc stats&quot;)</nobr><br>
Run time: $runtime seconds.</td>
<td class="copyright">onis is written 2000-2002 <nobr>by Florian \&quot;octo\&quot; Forster</nobr> <nobr>&lt;flo at verplant dot org&gt;</nobr><br>
Get the latest version from the <a href="http://verplant.org/onis/" target="_blank"><b>onis homepage</b></a></td>
</tr>
</table>

</td></tr></table>
EOF
}

# end the HTML of the site..
sub end
{
	print OUT <<EOF;
</div>

<!-- script written by $::AUTHOR -->
</body>
</html>
EOF
}

sub output
{
	foreach my $line (@_)
	{
		print OUT $line;
	}

	return 1;
}

sub escape
{
	my @retval = ();

	foreach (@_)
	{
		s/\&/\&amp;/g;
		s/"/\&quot;/g;
		s/</\&lt;/g;
		s/>/\&gt;/g;

		if (m#(?:(?:ftp|https?)://|www\.)[a-z0-9_\.-]+\.[a-z]{2,4}(?:\d+)?(?:/[\w\d\.\%/-]+)?(?=\W|$)#)
		{
			my $match = $&;
			my $prematch = $`;
			my $postmatch = $';
			
			if ($match =~ /^www/) { $match = 'http://' . $match; }
			if ($match !~ m#://.+/#) { $match .= '/'; }
			
			$_ = $prematch . qq(<a href="$match" target="_blank">$match</a>) . $postmatch;
		}

		push (@retval, $_);
	}

	if (wantarray ())
	{
		return @retval;
	}
	else
	{
		return join ("\n", @retval);
	}
}

END
{
	close (OUT);
}

1;
