#=========================================================#
# onis 0.3.3                             2002-03-29 11:59 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# parser/eggdrop.pm                                       #
#=========================================================#


package parser;

use strict;
use vars qw( $data );

require 'tools.pm';

*data = *::data;

sub parse
{
	my $line = shift;

# this looks like a normal line, so we add it to
# our hash (%$data) by using "input::add_line ();"
	if ($line =~ /^\[(\d\d):(\d\d)\]\s<([^>]+)>\s(.*)/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'text');
	}

# action
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\sAction:\s(\S+)\s(.*)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, "* $nick $text", $hour, $minute, 'action');
	}

# somebody joined our channel.. It's time to
# register his host and name..
	elsif ($line =~ /^\[\d\d:\d\d\]\s(\S+)\s\(([^\)]+)\)\sjoined\s(#.+)\.$/)
	{
		my ($nick, $host, $channel) = ($1, $2, $3);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
	}

# Yeah, we can see soembody's host.. Let's
# remember it.. :)
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s(#\S+):\smode\schange\s'([-+][^']+)'\sby\s([^!]+)!(.+)$/)
	{
		my ($hour, $minute, $channel, $mode, $nick, $host) = ($1, $2, $3, $4, $5, $6);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
		input::add_line ($nick, $mode, $hour, $minute, 'mode');
	}
	
# this gets called when somebody changes name:
# we just copy his data to a new nick..
	elsif ($line =~ /^\[\d\d:\d\d\]\sNick\schange:\s(\S+)\s->\s(\S+)$/)
	{
		my ($old_nick, $new_nick) = ($1, $2);
		$data->{'nick2host'}{$new_nick} = $data->{'nick2host'}{$old_nick}
	}

# topic
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\sTopic\schanged\son\s(#\S+)\sby\s([^!]+)!(\S+):\s(.+)$/)
	{
		my ($hour, $minute, $channel, $nick, $host, $text) = ($1, $2, $3, $4, $5, $6);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
		input::add_line ($nick, $text, $hour, $minute, 'topic');
	}

# quit
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s(\S+)\s\(([^\)]+)\)\sleft\sirc:\s(.+)$/)
	{
		my ($hour, $minute, $nick, $host, $text) = ($1, $2, $3, $4, $5);
		input::add_host ($nick . '!' . $host);
		input::add_line ($nick, $text, $hour, $minute, 'quit');
	}

# kick
# [02:03] zZz^sauri^zZz kicked from #bb by [BB]Stark: nur fur die statistik
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s(\S+)\skicked\sfrom\s(#\S+)\sby\s([^:]+):\s(.+)$/)
#	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s([^!]+)!(\S+)\skicked\sfrom\s(#\S+)\sby\s([^!]+)!([^:]+):\s(.+)$/)
	{
		my ($hour, $minute, $nick_received, $channel, $nick_given, $text) = ($1, $2, $3, $4, $5, $6, $7, $8);
#		input::add_host ($nick_received . '!' . $host_received);
#		input::add_host ($nick_given    . '!' . $host_given   );
		input::add_channel ($channel);
		input::add_line ($nick_received, $text, $hour, $minute, 'kick_received');
		input::add_line ($nick_given,    $text, $hour, $minute, 'kick_given'   );
	}
	
	elsif ($::debug)
	{
		print "Not parsed: $line\n";
	}
}

1;
