#=========================================================#
# onis 0.3.3                             2002-03-15 19:32 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# parser/ircii.pm                                         #
#=========================================================#

package parser;

use strict;

require 'tools.pm';

sub parse
{
	my $line = shift;
	
# this looks like a normal line, so we add it to
# our hash (%$data) by using "input::addline ();"
	if ($line =~ /^\[(\d\d):(\d\d)\]\s<([^>]+)>\s(?:\[[^\]]+\]\s)?(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'text');
	}

# someone's doing an action..
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s(\*\s(\S+)\s.+)$/)
	{
		my ($hour, $minute, $action, $nick) = ($1, $2, $3, $4);
		input::add_line ($nick, $action, $hour, $minute, 'action');
	}

# somebody joined our channel.. It's time to
# register his host and name..
	elsif ($line =~ /^\[\d\d:\d\d\]\s\*\*\*\s(\S+)\s\[([^\]]+)\]\shas\sjoined\schannel\s(#.+)$/)
	{
		my ($nick, $host, $channel) = ($1, $2, $3);
		input::add_channel ($channel);
		input::add_host ($nick . '!' . $host);
	}

# nickchange
	elsif ($line =~ /^\[\d\d:\d\d\]\s\*\*\*\s(\S+)\s\[([^\]]+)\]\sis\snow\sknown\sas\s(.+)$/)
	{
		input::add_host ($1 . '!' . $2);
		input::add_host ($3 . '!' . $2);
	}

# quit
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\*\*\sSignoff:\s(\S+)\s\[([^\]]+)\]\s\[([^\]]+)\]$/)
	{
		my ($hour, $minute, $nick, $host, $message) = ($1, $2, $3, $4, $5);
		input::add_host ($nick . '!' . $host);
		input::add_line ($nick, $message, $hour, $minute, 'quit');
	}

# leave
	#[21:32] *** max_ [arielle@cybercafe.franken.de] has left channel #cybercafe42.de
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\*\*\s(\S+)\s\[([^\]]+)\]\shas\sleft\schannel\s(#\S+)$/)
	{
		my ($hour, $minute, $nick, $host, $channel) = ($1, $2, $3, $4, $5);
		input::add_channel ($channel);
		input::add_host ($nick . '!' . $host);
	}
	
# kick
	elsif ($line =~ /^\*\*\*\s(\S+)\shas\sbeen\skicked\soff\schannel\s(#\S+)\sby\s(\S+)\s\(([^\)]+)\)/)
	{
		my ($bad_guy, $channel, $good_guy, $reason) = ($1, $2, $3, $4);
		input::add_channel ($channel);
		input::add_line ($bad_guy,  $reason, 0, 0, 'kick_received');
		input::add_line ($good_guy, $reason, 0, 0, 'kick_given'   );
	}

# topic
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\*\*\s(\S+)\s\[([^\]]+)\]\shas\schanged\sthe\stopic\son\schannel\s(#\S+)\sto\s(.+)$/)
	{
		my ($hour, $minute, $nick, $host, $channel, $text) = ($1, $2, $3, $4, $5, $6);
		input::add_channel ($channel);
		input::add_host ($nick . '!' . $host);
		input::add_line ($nick, $text, $hour, $minute, 'topic');
	}

# mode
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\*\*\sMode\schange\s"([^"]+)"\son\schannel\s(#\S+)\sby\s(\S+)\s\[([^\]]+)\]$/)
	{
		my ($hour, $minute, $mode, $channel, $nick, $host) = ($1, $2, $3, $4, $5, $6);
		input::add_channel ($channel);
		input::add_host ($nick . '!' . $host);
		input::add_line ($nick, $mode, $hour, $minute, 'mode');
	}	
	
	elsif ($::debug)
	{
		print 'Not parsed: ' . $line . "\n";
	}
}

1;
