#=========================================================#
# onis 0.3.3                             2002-03-15 19:48 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# parser/mirc.pm                                          #
#=========================================================#

package parser;

use strict;
use vars qw( $data );

require 'tools.pm';

*data = *::data;

sub parse
{
	my $line = shift;

# this looks like a normal line, so we add it to
# our hash (%$data) by using "input::addline ();"
	if ($line =~ /^\[(\d\d):(\d\d)\]\s<([^>]+)>\s(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'text');
	}

# somebody joined our channel.. It's time to
# register his host and name..
	elsif ($line =~ /^\[\d\d:\d\d\]\s\*\*\*\s(\S+)\s\((.+)\)\shas\sjoined\s(#.+)$/)
	{
		my ($nick, $host, $channel) = ($1, $2, $3);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
	}

# this gets called when somebody changes name:
# we just copy his data to a new nick..
	elsif ($line =~ /^\[\d\d:\d\d\]\s\*\*\*\s(\S+)\sis\snow\sknown\sas\s(\S+)$/)
	{
		$data->{'nick2host'}{$2} = $data->{'nick2host'}{$1};
	}

# somebody's leaving us..
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*{3}\s(.+?)\shas\sleft\sIRC\s\((.+)\)/)
	{
		my ($hour, $minute, $nick, $message) = ($1, $2, $3, $4);
		input::add_line ($nick, $message, $hour, $minute, 'quit');
	}

# leave
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\*\*\s(\S+)\s\(([^\)]+)\)\shas\sleft\s(#\S+)\s\((.+)\)$/)
	{
		my ($hour, $minute, $nick, $host, $channel, $text) = ($1, $2, $3, $4, $5, $6);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
		input::add_line ($nick, $text, $hour, $minute, 'quit');
	}

# someone's doing an action..
	elsif ($line =~ /^\[(\d\d):(\d\d)\]\s\*\s(\S+)\s(.+)$/)
	{
		my ($hour, $minute, $nick, $action) = ($1, $2, $3, $4);
		$action = "* $nick $action";
		input::add_line ($nick, $action, $hour, $minute, 'action');
	}

# uuhhhh, SOMEONE's in trouble ;)
# (he/she got kicked..)
	elsif ($line =~ /^\[(\d\d):(\d\d)\] \*{3} (\S+) was kicked by (\S+) \((.+)\)$/)
	{
		my ($hour, $minute, $bad_guy, $good_guy, $reason) = ($1, $2, $3, $4, $5);
		input::add_line ($bad_guy,  $reason, $hour, $minute, 'kick_received');
		input::add_line ($good_guy, $reason, $hour, $minute, 'kick_given'   );
	}

# topic
	elsif ($line =~ /^\[(\d\d):(\d\d)\] \*\*\*\s(\S+)\schanges\stopic\sto\s'(.+)'$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'topic');
	}

# mode
	elsif ($line =~ /^\[(\d\d):(\d\d)\] \*\*\*\s(\S+)\ssets\smode:\s([\+\-]\S\s\S+)$/)
	{
		my ($hour, $minute, $nick, $mode) = ($1, $2, $3, $4);
		input::add_line ($nick, $mode, $hour, $minute, 'mode');
	}
	
	elsif ($::debug)
	{
		print "Not parsed: $line\n";
	}
}

1;
