#=========================================================#
# onis 0.3.3                             2002-03-15 19:52 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# parser/perlbot.pm                                       #
#=========================================================#

package parser;

use strict;
use vars qw( $data );

require 'tools.pm';

*data = *::data;

sub parse
{
	my $line = shift;
	
# text
	if ($line =~ /^(\d\d):(\d\d):\d\d\s<([^>]+)>\s(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'text');
	}
# action
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s\*\s(\S+)\s(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		$text = "* $nick $text";
		input::add_line ($nick, $text, $hour, $minute, 'action');
	}

# nickchange
	elsif ($line =~ /^\d\d:\d\d:\d\d\s\[NICK\]\s(\S+)\schanged\snick\sto:\s(\S+)$/)
	{
		$data->{'nick2host'}{$2} = $data->{'nick2host'}{$1};
	}
	
# join
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s(\S+)\s\(([^\)]+)\)\sjoined\s(.+)$/)
	{
		my ($hour, $minute, $nick, $host, $channel) = ($1, $2, $3, $4, $5);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
	}

# part
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s(\S+)\s\(([^\)]+)\)\sleft\s(.+)$/)
	{
		my ($hour, $minute, $nick, $host, $channel) = ($1, $2, $3, $4, $5);
		input::add_host ($nick . '!' . $host);
		input::add_channel ($channel);
	}

# quit
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s\[QUIT\]\s(\S+)\squit:\s(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'quit');
	}
	
# topic
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s\[TOPIC\]\s(\S+):\s(.+)$/)
	{
		my ($hour, $minute, $nick, $text) = ($1, $2, $3, $4);
		input::add_line ($nick, $text, $hour, $minute, 'topic');
	}

# kick
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s\[KICK\]\s(\S+)\swas\skicked\sby\s(\S+)\s\((.+)\)$/)
	{
		my ($hour, $minute, $bad_boy, $good_guy, $text) = ($1, $2, $3, $4, $5);
		input::add_line ($bad_boy,  $text, $hour, $minute, 'kick_received');
		input::add_line ($good_guy, $text, $hour, $minute, 'kick_given');
	}

# mode
	elsif ($line =~ /^(\d\d):(\d\d):\d\d\s\[MODE\]\s(\S+)\sset\smode:\s([-+]\S+(?:\s.+)?)$/)
	{
		my ($hour, $minute, $nick, $mode) = ($1, $2, $3, $4);
		input::add_line ($nick, $mode, $hour, $minute, 'mode');
	}
	
	elsif ($::debug)
	{
		print "Not parsed: $line\n";
	}
}

1;
