#=========================================================#
# onis 0.3.3                             2002-03-15 19:34 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# parser/tools.pm                                         #
#=========================================================#

package input;

use strict;

use vars qw(
		$data
		$tmphosts
	   );

*data = *::data;
$tmphosts = 0;

sub add_host
{
	###########################
	# add a new host to $data #
	###########################

	my $user_host = shift;

	if (($data->{'config'}{'unsharp'}) and ($user_host =~ /\@[^\.]+\.[^\.]+\../))
	{
		unsharp_host (\$user_host) or return undef;
	}
	my ($nick, $host) = split (/!/, $user_host);
	$data->{'nick2host'}{$nick} = lc ($host);
}

sub add_channel
{
	my $channel = shift;

	return undef unless $channel;

	$channel =~ s/^#//;
	
	foreach (@{$::plugins{'channel'}})
	{
		&$_ ($channel, $channel, 0, 0, 'channel', 'foo');
	}

	return 1;
}

sub add_line
# let's add a lot of usefull information to $data.
# This routine will pass information on to the
# statistic modules WITHOUT checking, so you will
# have to check that before. I do it in the parse-
# modules and I recommend to do it the very same
# way in order to not cause confusion.
{
	my ($nick, $line, $hour, $minute, $type) = @_;
	return undef unless $line;
	$type = 'text' unless $type;
	
	return 0 unless defined $::plugins{$type};

	my $host;

	# check if we already know this nick's host
	if (defined $data->{'nick2host'}{$nick})
	{
		$host = $data->{'nick2host'}{$nick};
	}
	
	# uhm, no.. let's give him a number..
	else
	{
		$host = ++$tmphosts;
		$data->{'nick2host'}{$nick} = $host;
	}

# Is this nick excluded from statistics? It may be
# a bot or so.. We'll ignore it/him/her anyways
# and just return zero..
# Fester did this different, but in my opinion
# it's better to not count these nicks at all.
# Once a nick is excluded it should not modify the
# stats at all.
	if (defined $data->{'config'}{'excludelist'}{$host})
	{
		return 0;
	}

	foreach (@{$::plugins{$type}})
	{
		&$_ ($nick, $line, $hour, $minute, $type, $host);
	}
	
	return 1;
}

#
# The old unsharp_host was really messed up,
# so I did this re-write on August 17th.
#
sub unsharp_host
{
	my $host_ref = shift;
	return undef unless (ref $host_ref);

	my ($nick_and_user, $host) = split (/\@/, $$host_ref);

	if (!$host)
	{
		warn 'Weird hostname: ' . $$host_ref . "\n" if $main::debug;
		return undef;
	}

	my (@comp) = split (/\./, $host);
#
# HANDLING OF PURE IPS
#
# Here we handle IPs only. They're pretty easy
# and therefore easy to unsharp. Mode ! will
# assume that IPs never change and therefore
# won't change them, mode two and three will
# remove the last number so a whole net is put
# together
#
	if ($host =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)
# it's an IP, so lat's remove the last number
# so the who;e class C net would be the same..
# (but only if unsharp mode 2/3 is selected..)
# Example:
# "216.115.108.243" (yahoo.com)
#   becomes
# "216.115.108."
	{
		if ($data->{'config'}{'unsharp'} > 1)
		{
			$comp[3] = '';
		}
	}

#
# HANDLING OF "NORMAL" DOMAINS
#
# Ok, the host does not seem to be an IP (except
# IPv6 maybe, I'll do that later..). The
# different modes will use different (but
# alike) routines to detect dial-up-domains
# and unsharp them so the entire provider is
# recognized. Mode 1 is the "sharpest" and
# doesn't change the domains a whole lot,
# mode 3 removes a lot more so you should hope
# that your users won't change their ident alot..
#
	else
	{
		if ($data->{'config'}{'unsharp'} == 1)
# - M O D E   O N E -
#
# ... removes all numbers from the computer's
# name (= subdomain)
		{
			if ($comp[0] =~ /\d\D|\D\d/)
# if it contains both, numbers and non-numbers
# remove the numbers. Example:
# "dialin-198-040.provider.com"
#   becomes
# "dialin--.provider.com"
			{
				$comp[0] =~ s/\d//g;
			}
			elsif ($comp[0] =~ /^\d+$/)
# if it contains ONLY numbers we remove it all
# Example:
# "48990378.prodiver.com"
#   becomes
# ".provider.com"
			{
				$comp[0] = '';
			}
# the last option does not contain any numbers so
# we don't have to do anything..
		}

##################################################

		elsif ($data->{'config'}{'unsharp'} == 2)
# - M O D E   T W O -
#
# ... removes the entire subdomain if it contains
# any amoun of numbers.
		{
			if ($comp[0] =~ /\d/)
# If the subomain contains a number it gets
# removed. Example:
# "pD951C496.provider.com" (this is the sheme of
#			   one of the big german
#			   providers "t-online")
#   becomes
# ".provider.com"
			{
				$comp[0] = '';
			}
# If there is no number in the domain we will not
# edit anything since it seems like a non-dial-up
# domain and hopefully won't change.
		}

##################################################

		elsif ($data->{'config'}{'unsharp'} > 2)
# - M O D E   T H R E E -
#
# ... removes the entire subdomain, no matter
# how it looks like. Let's hope people won't
# change their idents since it's basically the
# only way to recognize two or more people with
# the same provider/domain
		{
			$comp[0] = '';
		}

		else
		{
			warn "unsharp_host was called but \$data->{'config'}{'unsharp'} did not apply to any routine. Check for programming errors! It might contain a floating point number..\n" if $main::debug;
		}
	}

	my $unsharp_host = $nick_and_user . '@' . join ('.', @comp);

	print '"' . $$host_ref . '" becames "' . $unsharp_host . '" with unsharp-mode ' . $data->{'config'}{'unsharp'} . "\n" if ($main::debug > 1);

	$$host_ref = $unsharp_host;
}

1;
