#=========================================================#
# onis 0.3.3                             2002-03-29 20:20 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# plugins/core.pm                                         #
#=========================================================#

package core;

use strict;

use vars qw( $data %color);

*data = *::data;
*color = *::color;

push (@{$::plugins{'text'}}, \&add);

push (@{$::plugins{'calculate'}}, \&calculate);
push (@{$::plugins{'output'}}, \&output);

return 1;

sub add
# averange line length
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	print "core::add ($nick, $line, $hour, $minute, $type, $host);\n" if $::debug > 1;
	
	++$data->{'nicksorted'}{$nick}{$host}{'general'};
	++$data->{'nicksorted'}{$nick}{$host}{'time'}{int ($hour)};
	++$data->{'time'}{'total'}[$hour];
	++$data->{'lines'}{'total'};

	return undef if $host =~ m/^\d+$/;
	
	if ((length ($line) > $Config::config->{'quote'}{'min'})
	and (length ($line) < $Config::config->{'quote'}{'max'}))
	{
		push (@{$data->{'quote'}{$host}}, $line);
	}
}

sub calculate
{
	print "Generating the final hash." if $::debug;
	
# First we loop through all the nicks that we
# have. We will have more than one nick for one
# person. That's why we determine the host that
# was used the most and add this nick (and the
# lines it wrote) to the host. This way even if
# somebody was chatting from an internet-cafe or
# at a friend's we will still recognize them.
# Very weird hosts or turned off host-unsharpening
# may be other reasons.
	foreach my $s_nick (keys %{$data->{'nicksorted'}})
	{
		my ($maxhost, $maxlines, $alllines) = ('', 0, 0);
		my %alltimes = ();

# Now we loop through every host a nick has had.
# The lines that were written with this host get
# added to $alllines, if it is the most active
# host we put it into $maxhost and it's written
# lines to $maxlines
		my @hosts = keys %{$data->{'nicksorted'}{$s_nick}};
		
		foreach my $s_host (@hosts)
		{
			next unless defined $data->{'nicksorted'}{$s_nick}{$s_host}{'general'};
			if ($data->{'nicksorted'}{$s_nick}{$s_host}{'general'} > $maxlines)
			{
				$maxlines = $data->{'nicksorted'}{$s_nick}{$s_host}{'general'};
				$maxhost = $s_host;
			}
			$alllines += $data->{'nicksorted'}{$s_nick}{$s_host}{'general'};

# We copy (add) the numbers of lines to a specific
# hour to %alltimes and add it later to $data. 
			foreach my $hour (keys %{$data->{'nicksorted'}{$s_nick}{$s_host}{'time'}})
			{
				$alltimes{$hour} += $data->{'nicksorted'}{$s_nick}{$s_host}{'time'}{$hour};
			}
		}
		$data->{'hostsorted'}{$maxhost}{$s_nick}{'general'} += $alllines;
		
		foreach my $hour (keys %alltimes)
		{
			$data->{'hostsorted'}{$maxhost}{$s_nick}{'time'}{$hour} = $alltimes{$hour};
		}
	}

	print '.' if $::debug;


# Ok, we have now lots of hosts with at least one
# nick associated with each. Now we have to find
# the most active nick for each host. This works
# the same way we found the most active host.
# Purpose of this activity is to choose "nick"
# over "nick_AW" or "nick_sleep" for example. A
# simple hack here and we can tell which host had
# the most nicks which is a (/one of my) favorite
# details..
	foreach my $s_host (sort keys %{$data->{'hostsorted'}})
	{
		my ($maxnick, $maxlines, $alllines) = ('', 0, 0);
		my %alltimes = ();
		
# Same as above. We loop through all nicks
# associated with the host we have in $s_host and
# find the most used nick by putting it into
# $maxnick and it's lines into $maxlines.
		my @nicks = keys %{$data->{'hostsorted'}{$s_host}};

		foreach my $s_nick (@nicks)
		{
			if ($data->{'hostsorted'}{$s_host}{$s_nick}{'general'} > $maxlines)
			{
				$maxlines = $data->{'hostsorted'}{$s_host}{$s_nick}{'general'};
				$maxnick = $s_nick;
			}
			$alllines += $data->{'hostsorted'}{$s_host}{$s_nick}{'general'};
			foreach my $hour (keys %{$data->{'hostsorted'}{$s_host}{$s_nick}{'time'}})
			{
				$alltimes{$hour} += $data->{'hostsorted'}{$s_host}{$s_nick}{'time'}{$hour};
			}
		}

# Needed by the plugins to be able to easily run through all nicks used by one
# person and adding all stats together..
		push (@::nicklist, $maxnick);
		$::allnicks{$maxnick} = [ @nicks ];
		
# Ok, if you are lost in the algorithms above just
# check a little higher. But now there's something
# new: We have a new key in $data calles
# "finalhash" and as you may have guessed there's
# all the data we will need (for the main
# functions). To save the host can only be used
# for debug-resons, at least I can't picture any
# stat using it..
		$data->{'finalhash'}{$maxnick}{'general'} += $alllines;
		$data->{'finalhash'}{$maxnick}{'mainhost'} = $s_host;

# adds a array with all used nicks to the hash.
# used to collect all the data the stats-modules
# and Fester's extensions collected.

		foreach my $hour (keys %alltimes)
		{
			$data->{'finalhash'}{$maxnick}{'time'}{$hour} = $alltimes{$hour};
		}
	}

	print "[DONE]\n" if $::debug;
}

sub output
{
	activetimes ();
	ranking ();
	print html::OUT qq(\n<hr width="95%">\n<h2>Output by Plugins</h2>\n);
}
	
# this subroutines doesn't take any arguments either (stupid me). It prints the
# daily usage to the file.
sub activetimes
{
	my $most_lines = 0; # the most lines that were written in one hour..
	my $total_lines = 0; # the total amount of lines we wrote..
	my ($i, $j); # used in for-loops
	my $factor = 0; # used to find a bar's height
	my $newline = ''; # buffer variable..
	
# this for loop looks for the most amount of lines in one hour and sets
# $most_lines
	for ($i = 0; $i <= 23; $i++)
	{
		next unless defined $data->{'time'}{'total'}[$i];
		if ($most_lines < $data->{'time'}{'total'}[$i])
		{
			$most_lines = $data->{'time'}{'total'}[$i];
		}
		$total_lines += $data->{'time'}{'total'}[$i];
	}

	$factor = ((130 - 1) / $most_lines); # used to find a bar's height

	print html::OUT qq(<h2>When do we actually talk here?</h2>\n<table border="0" cellspacing="1">\n<tr>\n);

# this for circles through the four colors. Each color represents six hours.
# (4 * 6 hours = 24 hours)
	for ($i = 0; $i <= 3; $i++)
	{
		for ($j = 0; $j <= 5; $j++)
		{
			my $k = (($i * 6) + $j);
			# $k is the hour we're printing (in the 24-hour-system)

			if (!defined $data->{'time'}{'total'}[$k]) { $data->{'time'}{'total'}[$k] = 0; }

			print html::OUT '<td class="bar" width="22">'
			. (int ($data->{'time'}{'total'}[$k] * 1000 / $total_lines) / 10)
			. '%<br><img src="'
			. $Config::config->{'bar'}{'vertical'}[$i]
			. '" width="18" height="'
			. (int ($data->{'time'}{'total'}[$k] * $factor) + 1)
			. qq("></td>\n);

			$newline .= qq(<td class="hour" width="22">$k</td>\n);
		}
	}

# $newline holds the entire 2nd line of the table. That's basically zero
# through 23..
	print html::OUT "</tr>\n<tr>\n$newline</tr>\n</table>\n<br>\n";
}

sub ranking
{
#
# This routine does not take any arguments since
# I was extremely stupid when I wrote this pro-
# gram and used global variables a lot.. I might
# change that later but that's more work than
# it's worth it right now. (For me anyways.. If
# you have a problem here go ahead and fix it :)
#
	my $count = 0; # $count is needed to check
	# wheather it's the first entry in which
	# case the maximum amount of lines is
	# saved. It's also needed to write a num-
	# ber to each line.

	my $factor; # needed by "&bar ();" to
	# determin the width of the lines.

	my $maxlines; # the maximum amount of
	# lines one person wrote (1st place..)

	my $linescount; # count how many people
	# are displayed. Needed to switch to the
	# short-mode or even stop displaying
	# names..

	my $longlines = $Config::config->{'longlines'};
	# declares how many people should be dis-
	# played with a random line and grafic
	# of theyr channel activity..

	my $shortlines = $Config::config->{'shortlines'};
	# same as $longlines but for the short table at
	# the end. Each lines contains *SIX* names..

# here we print the top of the table..	
print html::OUT <<EOF;
<hr width="95%">
<h2>Most active nicks:</h2>
<p>(Nicks sorted by number of lines written)</p>
<table nowrap border="0" cellspacing="1">
<tr>
<td class="invis">&nbsp;</td>
<td class="header">Nick</td>
<td class="header">Number of Lines</td>
<td class="header">Random Quote</td>
</tr>
EOF

# this foreach sorts and loops through all nicks
# in "$data->{'finalhash'}" and displays infor-
# mation about them..
	foreach my $nick (sort { $data->{'finalhash'}{$b}{'general'} <=> $data->{'finalhash'}{$a}{'general'} } keys %{$data->{'finalhash'}})
	{
		$linescount++;
		if ($linescount <= $longlines)
		# As long as we didn't hit the 
		# $longlines-limit we continue
		# our table..
		{
			print $linescount . ' ' . $nick . '!' . $data->{'finalhash'}{$nick}{'mainhost'} . "\n" if $::debug > 1;
			print html::OUT "<tr>\n  <td class=\"numeration\">" . ++$count . "</td>\n";
			if ($count == 1) { $maxlines = $data->{'finalhash'}{$nick}{'general'}; $factor = ((100 - 4) / $maxlines); }
			print html::OUT "  <td>$nick</td>\n";
			print html::OUT '  <td>';
			bar (\%{$data->{'finalhash'}{$nick}{'time'}}, $factor);

			print html::OUT $data->{'finalhash'}{$nick}{'general'} . "</td>\n";
			my $quote = $data->{'quote'}{$data->{'finalhash'}{$nick}{'mainhost'}}[int (rand (scalar @{$data->{'quote'}{$data->{'finalhash'}{$nick}{'mainhost'}}}))];
			if ($quote) { $quote = '"' . $quote . '"'; } else { $quote = '-- no quote available --'; }
			html::output ('  <td class="quote">', html::escape ($quote), "</td>\n");
			html::output ("</tr>\n");
			if ($linescount == $longlines) { print html::OUT '</table>' . "\n"; }
		}


		elsif (($linescount > $longlines) and ($linescount <= ($longlines + $shortlines * 6)))
		# Ok, we have too many people to
		# list them all so we start a
		# smaller table and just list the
		# names.. (Six names per line..)
		{
			if ($linescount == ($longlines + 1)) { print html::OUT "<h2>They didn't write so much:</h2>\n<table border=\"0\" cellspacing=\"1\">\n"; }
			if (((($linescount - ($longlines + 1)) / 6) == int (($linescount - ($longlines + 1)) / 6)) and ($linescount != ($longlines + 1))) { print html::OUT "</tr>\n<tr>"; }
			print html::OUT "<td>$nick ($data->{'finalhash'}{$nick}{'general'})</td>";
			if ($linescount == ($longlines + $shortlines * 6)) { print html::OUT "</tr>\n</table>\n"; }
		}

		# There is no else. There are
		# just too many people around.
		# I might add a "There are xyz
		# unmentioned nicks"-line..
	}
	print html::OUT "</table>\n";
}

# this is called by "&ranking ();" and prints the horizontal usage-bar in the
# detailed nick-table
sub bar
{
	my %source = %{$_[0]};
	my @splittimes;
	my $tmp;
	
# this for-loop circles through the four colors and prints the part of the bar
# in it's appropriate width.
	for (my $i = 0; $i <= 3; $i++)
	{

# this for adds the times of six hours and pushes it into an array.
		for (my $j = 0; $j <= 5; $j++)
		{
			$tmp = (($i * 6) + $j);
			if (defined $source{$tmp})
			{
				$splittimes[$i] += $source{$tmp};
			}
			elsif (!defined $splittimes[$i])
			{
				$splittimes[$i] = 0;
			}
		}
		print html::OUT '<img src="' . $Config::config->{'bar'}{'horizontal'}[$i] . '" height="15" width="' . int (($splittimes[$i] * $_[1]) + 1) . '" align=top>';
	}
}
