#=========================================================#
# onis 0.3.3                             2002-02-28 18:00 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# plugins/extensions.pm                                   #
#=========================================================#
package extensions;

use strict;

use vars qw( $data );

$data = $main::data;

1;

sub stat_add_long_lines #done
# averange line length
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	$data->{'nicksorted'}{$nick}{$host}{'llength'} += length ($line);
	$data->{'stats'}{'llength'}{'total'} += length ($line);
	$data->{'stats'}{'llength'}{'lines'}++;
}

sub stat_add_yell_percentage #done
# YELLS (all capital letters)
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	if ($line eq (uc $line) and $line =~ /\w{2}/)
	{
#		print "extensions.pm, line 28: yelled line: \"$line\"\n";
		$data->{'nicksorted'}{$nick}{$host}{'yells'}++;
	}
}

sub stat_add_soliloquies #done
# SOLILOQUIES (5 text lines in a row - actions don't count)
# The 'static' tree is used to keep a variable's value between functions.
# I could've used another global var, but I prefer to keep them to a minimum.
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	if ($data->{'static'}{'lastline'}{'nick'} eq $nick)
# somebody is talking AGAIN.. Let's get his count up..
	{
		$data->{'static'}{'lastline'}{'count'}++;
	}
	else
# someone else was talking.. So we have a new
# dandidate and reset the counter and remember his
# nick..
	{
		$data->{'static'}{'lastline'}{'nick'}  = $nick;
		$data->{'static'}{'lastline'}{'count'} = 1;
	}

	if ($data->{'static'}{'lastline'}{'count'} == 5)
# he/she did it and got five lines all by him/her
# self.. We count one soliloquy and reset the
# counetr..
	{
		$data->{'nicksorted'}{$nick}{$host}{'soliloquy'}++;
		$data->{'static'}{'lastline'}{'count'} = 0;
	}
}

sub stat_add_custom_quit_msgs #TODO
# UNIQUE QUITS
# Since it's *unique* quit messages, it makes it much more simple to simply use
# the message itself as the key.
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

# Make sure $line is non-empty
	return unless $line;

# I hope this brings some performance..
	my $lc_line = lc ($line);

# Filter out these messages.. I think it's faster
# this way than have the message being compared
# with an array of non-interesting messages..
	if (	$lc_line eq 'ping timeout' or
		$lc_line eq 'eof from client' or
		$lc_line eq 'connection reset by peer' or
		$lc_line eq 'no route to host' or
		$lc_line eq 'leaving' or
		$lc_line eq 'connection timed out' or
		$lc_line eq 'excess flood')
	{
		# do nothing..
	}
	else
	{
#		print "extensions.pm, line 92: quit message: \"$line\"\n";
		$data->{'nicksorted'}{$nick}{$host}{'uquits'}{$line} = 1;
	}
}

sub stat_add_actions #done
# ACTIONS
# Simple count of actions done by each nick
# Actions are stored in the actionex array for
# later pulling of examples
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

#	print "extensions.pm, line 105: action called: \"$line\"\n";
	push (@{$data->{'nicksorted'}{$nick}{$host}{'actionex'}}, $line);
}

sub stat_add_questions #done
# QUESTIONS
# Look for a line with a question mark following
# at least one character but after all the word
# characters. (Allows "What?" "What?!?!" and
# "What? " But not"I ? LIKE ? CAKE") Then,
# increment a counter.
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	if ($line =~ /\w[^\s\?]*\?\W*$/)
	{
#		print "extensions.pm, line 121: this looks like a question: \"$line\"\n";
		$data->{'nicksorted'}{$nick}{$host}{'questions'}++;
	}
}

sub stat_add_kicks #TODO
# KICKS
# Keeps track of kicks received and given.
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	if ($type =~ /received/i)
	{
		push (@{$data->{'nicksorted'}{$nick}{$host}{'kicks'}{'recieved'}{'examples'}}, $line);
	}
	elsif ($type =~ /given/i)
	{
		push (@{$data->{'nicksorted'}{$nick}{$host}{'kicks'}{'given'}{'examples'}}, $line);
	}
#	else
#	{
#		what the f are we doing here??
#	}
}

sub stat_add_most_used_words #done
# MOST USED WORDS
# Words greater than # of letters set in config
# file. Increments $data->{'words'}{theword}
# Initializes if necessary
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	my @words = split (/\s+/, $line);
	foreach my $cword (@words)
	{
		$cword =~ s/\W//g; # Strip trailing punctuation

		next unless (length ($cword) >= $data->{'config'}{'minwordlength'}); # check length..

		$cword = lc $cword;

		$data->{'words'}{$cword}{'count'}++;

		$data->{'words'}{$cword}{'lnick'} = $nick;
		$data->{'words'}{$cword}{'ldate'} = $data->{'time'}{'current'};

		my $shour = sprintf("%02c", $hour);
		my $sminute = sprintf("%02c", $minute);
		$data->{'words'}{$cword}{'ltime'} = "$hour:$minute";
	}
}

# # # # # # # # # # # # # # # # # # # # # # # # #

sub stat_calc_long_lines
# Creates the longest lines statistics from the
# data collected. To be run AFTER the creation of
# the finl hash..
{
	my %average_hash = ();

	# Iterates through every nick and calculates average = int(total characters / total lines)
	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		my $total_length = 0;
		my $total_lines  = $data->{'finalhash'}{$final_nick}{'general'};

		next unless ($total_lines > $data->{'config'}{'stats'}{'minlines'});

		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
				$total_length += $data->{'nicksorted'}{$used_nick}{$used_host}{'llength'};
			}
		}

#		if (!$total_lines) { $total_lines = 1; $total_length = 0; } # this should not happen..

		print "$final_nick has $total_lines lines and a total_length of $total_length\n" if $main::debug;

		my $average = (int ((10 * $total_length) / $total_lines) / 10);

if ($average < 5) { print "extensions.pm, line 204: $final_nick wrote $total_lines lines with a total length of $total_length, which gives an average of $average\n"; }

		$data->{'finalhash'}{$final_nick}{'stats'}{'llength'} = $average;

		$average_hash{$final_nick} = $average;
	}

	# Finds the overall channel average the same way
	my $total_lines  = $data->{'stats'}{'llength'}{'lines'};
	my $total_length = $data->{'stats'}{'llength'}{'total'};

	if (!$total_lines) { $total_lines = 1; $total_length = 0; } # this should not happen..

	my $average = (int ((10 * $total_length) / $total_lines) / 10);

	$data->{'stats'}{'llength'}{'average'} = $average;

	# Find the user with the highest/lowest average
	# Ok, this is a bit tricky so I'll explain it:
	# I let all the keys of %average_hash get
	# sorted by sort. The stuff in the middle 
	# takes care that it gets sorted by the value,
	# not the key. Since the value is the average
	# the first nick (yes, it returns nicks) in
	# list has the highest average, the last the
	# lowest. This is (afaik) the most efficient
	# way to do stuff like this..
	my ($highest, $lowest) = (sort { $average_hash{$b} <=> $average_hash{$a} } (keys %average_hash))[0,-1];

	$data->{'stats'}{'llength'}{'highest'} = $highest;	# this are both nicks.. You will have to check
	$data->{'stats'}{'llength'}{'lowest'}  = $lowest;	# $data->{'finalhash'}{$nick}{'stats'}{'llength'}
								# to get the number(s)..
}

sub stat_calc_yell_percentage
# Yell percent
# This works exactly the same way as the Line
# Length routine, except it finds both the
# largest, and the next-to-largest value.
{
	my %yell_hash = ();

	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		my $total_yells = 0;
		my $total_lines  = $data->{'finalhash'}{$final_nick}{'general'};

		next unless ($total_lines > $data->{'config'}{'stats'}{'minlines'});

		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
				$total_yells += $data->{'nicksorted'}{$used_nick}{$used_host}{'yells'};
			}       
		}       
		
		if (!$total_lines) { $total_lines = 1; $total_yells = 0; } # this should not happen..

#		print "extensions.pm, line 261: $final_nick yelled $total_yells times (out of $total_lines)\n";

		my $percent = int ((100 * $total_yells) / $total_lines);

		$data->{'finalhash'}{$final_nick}{'stats'}{'yells'} = $percent;
	
		$yell_hash{$final_nick} = $percent;
	}

	# For a description of this please look
	# at &stat_calc_long_lines ();
	my ($first, $second) = (sort { $yell_hash{$b} <=> $yell_hash{$a} } (keys %yell_hash))[0,1];

	$data->{'stats'}{'yells'}{'first'} = $first;
	$data->{'stats'}{'yells'}{'second'} = $second;
}

sub stat_calc_custom_quit_msgs
# Number of unique quit messages.
# Will save the nick to $data->{'stats'}
{
	my %quits_hash = ();

	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		next unless $final_nick;
		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
				foreach (keys %{$data->{'nicksorted'}{$used_nick}{$used_host}{'uquits'}})
				{
					$data->{'finalhash'}{$final_nick}{'stats'}{'uquits'}{$_} = 1;
				}
			}
		}
		$quits_hash{$final_nick} = scalar (keys %{$data->{'finalhash'}{$final_nick}{'stats'}{'uquits'}});
	}

	# For a description of this please look
	# at &stat_calc_long_lines ();
	my ($first) = (sort { $quits_hash{$b} <=> $quits_hash{$a} } (keys %quits_hash))[0];

	$data->{'stats'}{'uquits'} = $first;
}

sub stat_calc_actions
# Count each user's actions and find the one
# with the most activity going on.
{
	my %action_hash = ();

	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		next unless ($data->{'finalhash'}{$final_nick}{'general'} > $data->{'config'}{'stats'}{'minlines'});
		
		my @actions = ();
		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
				next unless ref ($data->{'nicksorted'}{$used_nick}{$used_host}{'actionex'});
				push (@actions, @{$data->{'nicksorted'}{$used_nick}{$used_host}{'actionex'}});
			}
		}

		my $total_actions = scalar @actions;

		@{$data->{'finalhash'}{$final_nick}{'stats'}{'actions'}} = @actions;

		$action_hash{$final_nick} = $total_actions;
	}

	# For a description of this please look
	# at &stat_calc_long_lines ();
	my ($first) = (sort { $action_hash{$b} <=> $action_hash{$a} } (keys %action_hash))[0];

	$data->{'stats'}{'action'} = $first;
}

#{
#        my ($nick, $line, $hour, $minute, $type, $host) = @_;
#
#        if ($line =~ /\w[^\s\?]*\?\W*$/)
#        {
#                print "extensions.pm, line 121: this looks like a question: \"$line\"\n";
#                $data->{'nicksorted'}{$nick}{$host}{'questions'}++;
#        }
#}
sub stat_calc_questions
# Count all the questions a user asked..
{
	my %question_hash = ();
	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		# check for minlines
		next unless ($data->{'finalhash'}{$final_nick}{'general'} > $data->{'config'}{'stats'}{'minlines'});

		my $num_questions = 0;
		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
				$num_questions += $data->{'nicksorted'}{$used_nick}{$used_host}{'questions'};
			}
		}
		$question_hash{$final_nick} = $num_questions;
	}

	my ($first, $second) = (sort { $question_hash{$b} <=> $question_hash{$a} } (keys %question_hash))[0,1];

	$data->{'stats'}{'questions'} = { $first => $question_hash{$first}, $second => $question_hash{$second} };
}

sub stat_calc_kicks
# 2001-12-03
# prepare the collected information about given and received kicks
{
#                push (@{$data->{'nicksorted'}{$nick}{$host}{'kicks'}{'recieved'}{'examples'}}, $line);
#                push (@{$data->{'nicksorted'}{$nick}{$host}{'kicks'}{'given'}{'examples'}}, $line);
	my %received_kicks_hash = ();
	my %given_kicks_hash = ();
	foreach my $final_nick (keys %{$data->{'finalhash'}})
	{
		# check for minlines
		next unless ($data->{'finalhash'}{$final_nick}{'general'} > $data->{'config'}{'stats'}{'minlines'});

		my $num_given_kicks = 0;
		my $num_received_kicks = 0;
		foreach my $used_nick (@{$data->{'finalhash'}{$final_nick}{'nicklist'}})
		{
			foreach my $used_host (keys %{$data->{'nicksorted'}{$used_nick}})
			{
# count the kicks received
				if (defined $data->{'nicksorted'}{$used_nick}{$used_host}{'kicks'}{'recieved'})
				{
					$num_received_kicks += scalar @{$data->{'nicksorted'}{$used_nick}{$used_host}{'kicks'}{'recieved'}{'examples'}};
				}
# count the given kicks, too
				if (defined $data->{'nicksorted'}{$used_nick}{$used_host}{'kicks'}{'given'})
				{
					$num_given_kicks += scalar @{$data->{'nicksorted'}{$used_nick}{$used_host}{'kicks'}{'given'}{'examples'}};
				}
			}
		}
		$received_kicks_hash{$final_nick} = $num_received_kicks;
		$given_kicks_hash{$final_nick} = $num_given_kicks;
	}

	my ($first_given, $second_given) = (sort { $given_kicks_hash{$b} <=> $given_kicks_hash{$a} } (keys %given_kicks_hash))[0,1];
	my ($first_received, $second_received) = (sort { $received_kicks_hash{$b} <=> $received_kicks_hash{$a} } (keys %received_kicks_hash))[0,1];

#	HERE IS A LOT TODO!!!
die;
}
# # # # # # # # # # # # # # # # # # # # # # # # #

sub stat_print_long_lines
{
	my $channel_average = $data->{'stats'}{'llength'}{'average'};
	my $highest_nick = $data->{'stats'}{'llength'}{'highest'};
	my $highest_average = $data->{'finalhash'}{$highest_nick}{'stats'}{'llength'};
	my $lowest_nick = $data->{'stats'}{'llength'}{'lowest'};
	my $lowest_average = $data->{'finalhash'}{$lowest_nick}{'stats'}{'llength'};

	print html::out "<p>Channel average: $channel_average..<br>\n";
	print html::out "Longest lines by $highest_nick (average: $highest_average)<br>\n";
	print html::out "Shortest lines by $lowest_nick (average: $lowest_average)</p>\n\n";
}

sub stat_print_yell_percentage
{
	my $first_nick  = $data->{'stats'}{'yells'}{'first'};
	my $second_nick = $data->{'stats'}{'yells'}{'second'};

	my $first_percent  = $data->{'finalhash'}{$first_nick }{'stats'}{'yells'};
	my $second_percent = $data->{'finalhash'}{$second_nick}{'stats'}{'yells'};

	print html::out "<p>$first_nick yelled the most ($first_percent\% of all his lines..)<br>\n";
	print html::out "the next is $second_nick with $second_percent percent of all his lines yelled..</p>\n\n";
}

sub stat_print_custom_quit_msgs
{
	my $nick = $data->{'stats'}{'uquits'};
	my $number = scalar (keys %{$data->{'finalhash'}{$nick}{'stats'}{'uquits'}});
	my $example = (keys %{$data->{'finalhash'}{$nick}{'stats'}{'uquits'}})[int (rand ($number))];

	print html::out "<p>$nick had the most unique quit messages. $number different that is. Here's an example of this masterpiece:<br>\n";
	print html::out "\&quot;$example\&quot;</p>\n\n";
}

sub stat_print_actions
{
	my $nick = $data->{'stats'}{'action'};
	my $number = scalar @{$data->{'finalhash'}{$nick}{'stats'}{'actions'}};
	my $example = ${$data->{'finalhash'}{$nick}{'stats'}{'actions'}}[int (rand ($number))];

	print html::out "<p>$nick had the most action going on.. Whole $number times. Here's an example:<br>\n";
	print html::out "\&quot;$example\&quot;</p>\n\n";
}

sub stat_print_questions
{
	my ($first, $second) = (sort { $data->{'stats'}{'questions'}{$b} <=> $data->{'stats'}{'questions'}{$a} } (keys %{$data->{'stats'}{'questions'}}));
	my ($f_questions, $s_questions) = ($data->{'stats'}{'questions'}{$first}, $data->{'stats'}{'questions'}{$second});

	print html::out "<p>$first asked the most questions.. ($f_questions total)<br>\n";
	print html::out "$second with $s_questions was still very interested</p>\n\n";
}
