#=========================================================#
# onis 0.3.3                             2002-03-29 18:32 #
# http://verplant.org/onis/                               #
# Florian Forster <flo@verplant.org>                      #
#                                                         #
# plugins/words.pm                                        #
#=========================================================#

package words;

use strict;

use vars qw(
		$data $_data %config
		@words @counts
		@usedby
	);

*data = *::data;
$_data = {};
%config = Config->get or return undef;

push (@{$::plugins{'text'}}, \&add);

push (@{$::plugins{'calculate'}}, \&calculate);
push (@{$::plugins{'output'}}, \&output);

sub add
{
	my ($nick, $line, $hour, $minute, $type, $host) = @_;

	print "words::add ($nick, $line, $hour, $minute, $type, $host);\n" if $::debug > 1;
	
	my @words = split (/\s+/, $line);
	foreach my $cword (@words)
	{
		# Strip trailing punctuation
		$cword =~ s/^[\W_]+|[\W_]+$//g;

		next if $cword =~ m/\W/;
		
		# check length..
		next unless (length ($cword) >= $config{'minlength'});

		# forget the case..
		$cword = lc $cword;

		++$_data->{$cword}{'count'};
		$_data->{$cword}{'last_used_by'} = $nick;
	}
}

sub calculate
{
	@words = ();
	
	my @tmp = (sort { $_data->{$b}{'count'} <=> $_data->{$a}{'count'} } (keys %$_data));
	foreach (@tmp) { if (!defined $data->{'nick2host'}{$_}) { push (@words, $_); } }

	if (scalar @words > $config{'amount'}) { splice (@words, $config{'amount'}); }

	@counts = ();
	@usedby = ();

	foreach (@words)
	{
		if (defined $data->{'nick2host'}{$_}) { next; }

		push (@counts, $_data->{$_}{'count'});
		push (@usedby, $_data->{$_}{'last_used_by'});
	}
}

sub output
{
	return undef unless scalar @counts;
	print html::OUT <<EOF;
<div>
<table border="0" cellspacing="1" width="400">
<tr>
<td class="invis">&nbsp;</td>
<td class="header">Word</td>
<td class="header">Times used</td>
<td class="header">Last used by</td>
</tr>
EOF

	my $i = 0;
	foreach my $w (@words)
	{
		++$i;
		my $count = shift (@counts);
		my $by = shift (@usedby);
		
		print html::OUT "<tr>\n";
		print html::OUT "  <td class=\"numeration\">$i</td>\n";
		print html::OUT "  <td>$w</td>\n";
		print html::OUT "  <td>$count</td>\n";
		print html::OUT "  <td>$by</td>\n";
		print html::OUT "</tr>\n";
	}
	
	print html::OUT <<EOF;
</table>
</div>
<br>

EOF
}

1;
